/*****************************************************************************
 * file.c: misc file checking functions
 *****************************************************************************
 * $Id: file.c,v 1.5 2003/07/28 22:33:41 alainjj Exp $
 *****************************************************************************
 * Copyright (C) 2001 Keuleu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 *****************************************************************************/
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>
#include "config.h"
#include "file.h"

extern int errno;

char user_dir[255];

#ifdef MISS_MKDTEMP
char *mkdtemp (char *template)
{
  char *dir;
  dir = mktemp(template);
  if(dir == NULL) return NULL;
  if(mkdir(dir,0700) == -1) return NULL;
  return dir;
}
#endif

int
file_exist (char *name)
{
  struct stat buf;

  if (stat (name, &buf) == 0)
    return TRUE;
  else if (errno == ENOENT)
    return FALSE;
  else
    return TRUE;
}

void
check_user_directory (void)
{
  struct stat buf;
  char *tmp;
  int n;
  int had_to_create = 0;

  /* Check for user directory */
  sprintf (user_dir, "%s/.xawdecode", getenv ("HOME"));

  if (stat (user_dir, &buf) == 0)
    {
      // is it a regular file or a link ?
      if (S_ISREG (buf.st_mode) || S_ISLNK (buf.st_mode))
        {
	  char user_dir_tmp[255];
	  sprintf (user_dir_tmp, "%s/.xawdecode.XXXXXX", getenv ("HOME"));
	  
          fprintf (stderr,
                   "Looks like %s is a file (or a symlink) and not a directory.\n",
                   user_dir);
          fprintf (stderr, "Moving %s to %s/xawdecoderc\n", user_dir,
                   user_dir);
	  tmp = mkdtemp(user_dir_tmp);
	  chmod(tmp,S_IRWXU);
	  n=strlen(tmp);
	  strcat (tmp, "/xawdecoderc");
          rename (user_dir, tmp);
	  tmp[n]=0;
          if (rename (tmp, user_dir) != 0)
            {
              fprintf (stderr,
                       "Error (not fatal): coud not create xawdecode user directory\n");
	      tmp[n]=' ';
	      rename(tmp, user_dir);
	      tmp[n]='\0';
	      rmdir(tmp);
            }
          else
            {
              had_to_create = 1;
            }
        }
    }
  else
    {
      fprintf (stderr,
               "Warning: xawdecode user directory does not exist, I'm creating it.\n");
      if (mkdir (user_dir, S_IRWXU) != 0)
        fprintf (stderr,
                 "Error (not fatal): coud not create xawdecode user directory\n");
      else
        had_to_create = 1;
    }

  if (had_to_create)
    {
      fprintf (stderr, "I had to create the new xawdecode storage area!\n");
      fprintf (stderr,
               "If this is the very first time you are using xawdecode then don't worry,\n");
      fprintf (stderr, "this is a normal procedure.\n");
      fprintf (stderr,
               "If you used to use an older xawdecode version, then you may have some\n");
      fprintf (stderr,
               "interesting files in probably /usr/local/share/xawdecode.\n");
      fprintf (stderr,
               "If this the case, please copy all those files to the new\n");
      fprintf (stderr,
               "%s directory and remove /usr/local/share/xawdecode as root.\n",
               user_dir);
    }
}
